var fs = require("fs");
var path = require("path");
var url = require("url");
var http = require("http");
var https = require("https");
var host = "localhost";
var port = 80;
var hosts = {
	"api2.musical.ly": null,
	"api.tiktokv.com": null,
	"api-t2.tiktokv.com": null,
	"api-tt.hypstar.com": null,
	"api-t.tiktok.com": null,
	"api-m.tiktok.com": null,
	"m.tiktok.com": null,
	"www.tiktok.com": null
};
switch (process.argv.length) {
	case 4:
		port = +process.argv[3] || 80;
	case 3:
		host = process.argv[2]
}
function requestGet(uri, headers, callback) {
	try {
		var u = typeof uri === "string" ? new url.URL(uri) : uri, h, r;
		if (u.protocol === "http:") {
			h = http;
		} else if (u.protocol === "https:") {
			h = https;
		} else {
			process.nextTick(callback, new Error("Only http and https is supported."));
			return;
		}
		r = h.request({
			method: "GET",
			host: u.host,
			port: +u.port || 0,
			path: u.pathname + u.search,//+ u.hash
			headers: headers
		}, function(response) {
			if (callback === null) {
				return;
			} else if (response.statusCode >= 300 && response.statusCode < 400) {
				var l = response.headers["location"];
				if (l === undefined) {
					callback(new Error("Redirection without location."));
					callback = null;
				} else {
					try {
						requestGet(new URL(l, String(uri)), headers, callback);
					} catch (e) {
						callback(e);
						callback = null;
					}
				}
				return;
			} else if (response.statusCode !== 200) {
				callback(new Error(response.statusMessage));
				callback = null;
				return;
			}
			callback(null, response, r, String(uri));
			callback = null;
		}).on("error", function(error) {
			if (callback === null) {
				return;
			}
			callback(error);
			callback = null;
		}).end();
	} catch (e) {
		process.nextTick(callback, e);
		return;
	}
}
function downloadAweme(filename, uri, headers, callback) {
	var f = function(error, repeat) {
		if (repeat > 3) {
			callback(error);
			return;
		}
		requestGet(uri, headers, function(error, response, request) {//"Referer": "https://www.tiktok.com/"
			if (error !== null) {
				f(error, repeat + 1);
				return;
			} else if (+response.headers["content-length"] === 0) {
				f(new Error("Response body is empty."), repeat + 1);
				return;
			}
			response.pipe(fs.createWriteStream(filename, {emitClose: true}).on("error", function(error) {
				if (callback === null) {
					return;
				}
				var f = callback;
				callback = null;
				fs.unlink(filename, function(error_) {
					f(error);
				});
			}).on("close", function() {
				if (callback === null) {
					return;
				}
				var ff = callback;
				callback = null;
				fs.stat(filename, function(error, stats) {
					if (error !== null) {
						f(error, repeat + 1);
						return;
					} else if (stats.size === 0) {
						fs.unlink(filename, function(error) {
							f(new Error("File is empty."), repeat + 1);
						});
						return;
					}
					ff(null);
				});
			}));
		});
	};
	f(null, 0);
}
try {
	if (!fs.lstatSync(__dirname + path.sep + "download").isDirectory()) {
		throw new Error("Path is not directory.");
	}
} catch (e) {
	if (e.code !== "ENOENT") {
		console.error(e);
		return;	
	}
	try {
		fs.mkdirSync(__dirname + path.sep + "download");
	} catch (e) {
		console.error(e);
		return;
	}
}
process.chdir(__dirname + path.sep + "download");
http.createServer(function(request, response) {
	var i, d;
	if (request.method === "GET") {
		if (request.url.length > 1 && request.url.charCodeAt(1) !== 63) {
			i = request.url.indexOf("?");
			d = request.url.substring(0, i > -1 ? i : request.url.length);
			if (d === "/aweme/v1/play/") {
				requestGet("http://api.tiktokv.com" + request.url, {"User-Agent": (d = request.headers["x-user-agent"]) !== undefined ? d : "okhttp"}, function(error, response_, request, uri) {
					if (error !== null) {
						response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
						response.end(http.STATUS_CODES[400]);
						return;
					}
					response.writeHead(302, http.STATUS_CODES[302], {"Location": uri});
					response.end();
				});
				return;
			}
			response.writeHead(404, {"Content-Type": "text/plain; charset=utf-8"});
			response.end(http.STATUS_CODES[404]);
			return;
		}
		fs.open("../index.htm", "r", function(error, fd) {
			if (error !== null) {
				console.error(error);
				response.writeHead(500, {"Content-Type": "text/plain; charset=utf-8"});
				response.end(http.STATUS_CODES[500]);
				return;
			}
			fs.fstat(fd, function(error, stats) {
				if (error !== null) {
					fs.close(fd, function() {});
					console.error(error);
					response.writeHead(500, {"Content-Type": "text/plain; charset=utf-8"});
					response.end(http.STATUS_CODES[500]);
					return;
				}
				var t = "\"" + (stats.mtimeMs - stats.mtimeMs % 1).toString(16) + "\"";
				if (request.headers["if-none-match"] !== undefined && request.headers["if-none-match"].indexOf(t) > -1) {
					fs.close(fd, function() {});
					response.writeHead(304, http.STATUS_CODES[304], {
						"Cache-Control": "public, max-age=3600",
						"ETag": t,
						"Access-Control-Allow-Origin": "*"
					});
					response.end();
					return;
				}
				response.writeHead(200, {
					"Content-Type": "text/html; charset=utf-8",
					"Cache-Control": "public, max-age=3600",
					"ETag": t,
					"Access-Control-Allow-Origin": "*"
				});
				fs.createReadStream(null, {fd: fd, encoding: "utf8"}).pipe(response);
			});
		});
	} else if (request.method === "POST") {
		if (request.url.length > 1 && request.url.indexOf("?") !== 1) {
			response.writeHead(404, {"Content-Type": "text/plain; charset=utf-8"});
			response.end(http.STATUS_CODES[404]);
			return;
		}
		d = "";
		request.setEncoding("utf8");
		request.on("data", function(data) {
			d += data;
			if (d.length > 2048) {
				request.destroy(new Error("Post data is too long."));
				return;
			}
		}).on("error", function(error) {
			console.error(error);
			response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
			response.end(http.STATUS_CODES[400]);
		}).on("end", function() {
			var f, h, u;
			try {
				u = url.parse(d);
			} catch (e) {
				response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
				response.end(http.STATUS_CODES[400]);
				return;
			}
			if (hosts[u.host] !== undefined) {
				h = {"User-Agent": (f = request.headers["x-user-agent"]) !== undefined ? f : "okhttp"};
			} else {
				response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
				response.end(http.STATUS_CODES[400]);
				return;
			}
			if ((f = request.headers["x-referer"]) !== undefined) {
				h["Referer"] = f;
			}
			if ((f = request.headers["x-cookie"]) !== undefined) {
				h["Cookie"] = f;
			}
			if ((f = request.headers["x-forwarded-for"]) !== undefined) {
				h["X-Forwarded-For"] = f + (request.connection.remoteAddress !== request.connection.localAddress ? ", " + request.connection.remoteAddress + ", " : ", ") + request.connection.localAddress;
			} else if (request.connection.remoteAddress !== request.connection.localAddress) {
				h["X-Forwarded-For"] = request.connection.remoteAddress + ", " + request.connection.localAddress;
			}
			if ((f = request.headers["accept-encoding"]) !== undefined && f.indexOf("gzip") > -1) {
				h["Accept-Encoding"] = f = "gzip";
			} else {
				f = "";
			}
			requestGet(u, h, function(error, response_) {
				if (error !== null) {
					console.error(error);
					response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
					response.end(http.STATUS_CODES[400]);
					return;
				} else if (response_.statusCode !== 200) {
					console.error(new Error(response_.statusMessage));
					response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
					response.end(http.STATUS_CODES[400]);
					return;
				}
				if (f.length > 0 && (f = response_.headers["content-encoding"]) !== undefined) {
					response.setHeader("Content-Encoding", f);
				}
				if ((f = response_.headers["content-type"]) !== undefined) {
					response.setHeader("Content-Type", f);
				}
				if ((f = response_.headers["content-length"]) !== undefined) {
					response.setHeader("Content-Length", f);
				}
				response.writeHead(200);
				response_.pipe(response);
			});
		});
	} else if (request.method === "PUT") {
		if (request.url.length > 1 && request.url.indexOf("?") !== 1) {
			response.writeHead(404, {"Content-Type": "text/plain; charset=utf-8"});
			response.end(http.STATUS_CODES[404]);
			return;
		}
		d = "";
		request.setEncoding("utf8");
		request.on("data", function(data) {
			d += data;
			if (d.length > 10000000) {
				request.destroy(new Error("Post data is too long."));
				return;
			}
		}).on("error", function(error) {
			console.error(error);
			response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
			response.end(http.STATUS_CODES[400]);
		}).on("end", function() {
			var i, e, v, f0, f1, f, h = {"User-Agent": "okhttp"};
			try {
				v = JSON.parse(d);
				if (typeof v !== "object" || v.constructor !== Array) {
					throw new Error("Unsupported type detected.");
				}
			} catch (e) {
				response.writeHead(400, {"Content-Type": "text/plain; charset=utf-8"});
				response.end(http.STATUS_CODES[400]);
				return;
			}
			if ((f = request.headers["x-referer"]) !== undefined) {
				h["Referer"] = f;
			}
			if ((f = request.headers["x-cookie"]) !== undefined) {
				h["Cookie"] = f;
			}
			response.writeHead(200, {"Content-Type": "text/plain; charset=utf-8"});
			response.end();
			f1 = function(video) {
				fs.access(video.id + ".mp4", function(error) {
					if (error === null) {
						e--;
						f0();
						return;
					} else if (error.code !== "ENOENT") {
						console.log(video.id + " failed");
						console.error(error);
						e--;
						f0();
						return;
					}
					downloadAweme(video.id + ".mp4", video.uri, h, function(error) {
						var i;
						if (error !== null) {
							console.log(video.id + " failed");
							console.error(error);
						}
						e--;
						f0();
					}, 0);
				});
			};
			i = 0;
			e = 0;
			f0 = function() {
				while (i < v.length && e < 20) {
					if (typeof v[i].id === "string" && typeof v[i].uri === "string") {
						e++;
						f1(v[i]);
					}
					i++;
				}
			};
			f0();
		});
	} else {
		response.writeHead(405, {"Content-Type": "text/plain; charset=utf-8"});
		response.end(http.STATUS_CODES[405]);
	}
}).listen(port, host, function() {
	console.log(this.address());
});