//csc.exe /target:winexe /optimize fetch.cs
//.NET 4.8
using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Text;
using System.Globalization;
using System.Collections.Generic;
using System.Net;
//using System.Net.Security;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Threading;

namespace AwemerApp {
	[DataContract]
	public class UrlData {
		[DataMember(Name = "url_list")]
		public string[] UrlList;
	}
	[DataContract]
	public class VideoData {
		[DataMember(Name = "play_addr")]
		public UrlData PlayAddr;
		//[DataMember(Name = "play_addr_264")]
		//public UrlData PlayAddr264;
		[DataMember(Name = "play_addr_265")]
		public UrlData PlayAddr265;
		[DataMember(Name = "play_addr_bytevc1")]
		public UrlData PlayAddrBytevc1;
	}
	[DataContract]
	public class AwemeDetailData {
		[DataMember(Name = "aweme_id")]
		public string AwemeId;
		[DataMember(Name = "create_time")]
		public long CreateTime;
		[DataMember(Name = "desc")]
		public string Desc;
		[DataMember(Name = "video")]
		public VideoData Video;
	}
	[DataContract]
	public class UserVideosData {
		//[DataMember(Name = "status_code")]
		//public long StatusCode;
		//[DataMember(Name = "has_more")]
		//public int HasMore;
		//[DataMember(Name = "max_cursor")]
		//public long MaxCursor;
		[DataMember(Name = "aweme_list")]
		public AwemeDetailData[] AwemeList;
	}
	[DataContract]
	public class MusicVideosData {
		//[DataMember(Name = "status_code")]
		//public long StatusCode;
		//[DataMember(Name = "has_more")]
		//public int HasMore;
		//[DataMember(Name = "cursor")]
		//public long Cursor;
		[DataMember(Name = "aweme_list")]
		public AwemeDetailData[] AwemeList;
	}
	[DataContract]
	public class HashtagVideosData {
		//[DataMember(Name = "status_code")]
		//public long StatusCode;
		//[DataMember(Name = "has_more")]
		//public int HasMore;
		//[DataMember(Name = "cursor")]
		//public long Cursor;
		[DataMember(Name = "aweme_list")]
		public AwemeDetailData[] AwemeList;
	}
	[DataContract]
	public class VideosData {
		//[DataMember(Name = "status_code")]
		//public long StatusCode;
		[DataMember(Name = "aweme_details")]
		public AwemeDetailData[] AwemeDetails;
	}
	public struct Aweme {
		public string Id;
		public long Date;
		public string Description;
		public string Url;
		public bool AddDate;
		public bool AddDescription;
	}
	public struct Download {
		public string Filename;
		public long Limit;
	}
	public struct JsonIndex {
		public char Type;//'n', 'f', 't', '"', '0', '[', '{'
		public int Index;
		public int Length;
	}
	/*public class UnexpectedException: Exception {
		public UnexpectedException() {}
		public UnexpectedException(string message): base(message) {}
		public UnexpectedException(string message, Exception inner): base(message, inner) {}
	}*/
	public class AwemeWebClient: WebClient {
		protected override WebResponse GetWebResponse(WebRequest request) {
			(request as HttpWebRequest).AllowAutoRedirect = true;
			HttpWebResponse r = base.GetWebResponse(request) as HttpWebResponse;
			if (r.ContentLength == 0) {
				throw new Exception("Response body is empty.");
			}
			return r;
		}
		/*public bool DownloadNonEmptyFile(Uri uri, string filename, byte[] buffer) {
			int i;
			byte[] b;
			Stream s;
			FileStream f;
			using (s = OpenRead(uri)) {
				if ((i = s.ReadByte()) == -1) {
					return false;
				}
				b = buffer != null ? buffer : new byte[1024];
				using (f = new FileStream(filename, FileMode.Create)) {
					f.WriteByte((byte)i);
					while ((i = s.Read(b, 0, b.Length)) > 0) {
						f.Write(b, 0, i);
					}
					f.Flush();
				}
			}
			return true;
		}
		public bool DownloadNonEmptyFile(string uri, string filename, byte[] buffer) {
			return DownloadNonEmptyFile(new Uri(uri), filename, buffer);
		}*/
	}
	public class MainForm: Form {
		public string Root;
		public string Dir;
		public bool Stop;
		public bool Date;
		public bool Description;
		public bool Quality;
		public bool Hevc;
		//public string Ip = "";
		public int Count;
		public FileStream Logger = null;
		public Graphics Graphics = null;
		public IContainer Components = new Container();
		public ToolTip Tip = new ToolTip();
		public FolderBrowserDialog DirOpener = new FolderBrowserDialog();
		public OpenFileDialog FileOpener = new OpenFileDialog();
		public CheckBox Log = new CheckBox();
		public CheckBox AddDate = new CheckBox();
		public CheckBox AddDescription = new CheckBox();
		public CheckBox ImproveQuality = new CheckBox();
		public CheckBox PreferHevc = new CheckBox();
		public Label DirLabel = new Label();
		public TextBox DirInput = new TextBox();
		public Button OpenDir = new Button();
		public Label TypeLabel = new Label();
		public ComboBox TypeInput = new ComboBox();
		public Label FileLabel = new Label();
		public TextBox FileInput = new TextBox();
		public Button OpenFile = new Button();
		public Label LimitLabel = new Label();
		public TextBox LimitInput = new TextBox();
		public Button Download = new Button();
		public TextBox Output = new TextBox();
		public readonly object StopLock = new Object();
		public readonly object OutLock = new Object();
		public readonly object SyncLock = new Object();
		public MainForm(string root, string directory) {
			Root = root;
			Dir = directory;
			SuspendLayout();
			Components.Add(Tip);
			Components.Add(DirOpener);
			Components.Add(FileOpener);
			Graphics = CreateGraphics();
			int t, w = (int)Graphics.MeasureString("directory:", DirLabel.Font).Width + 5;
			DirOpener.RootFolder = Environment.SpecialFolder.Desktop;
			DirOpener.SelectedPath = Root + Path.DirectorySeparatorChar + Dir;
			FileOpener.Title = "Select a file";
			FileOpener.RestoreDirectory = true;
			FileOpener.Filter = "Text file (*.txt)|*.txt|Any file (*.*)|*.*";
			FileOpener.InitialDirectory = Root;
			Log.AutoSize = false;
			Log.Size = new Size((int)Graphics.MeasureString("log", Log.Font).Width + 24, 21);
			Log.Location = new Point(5, t = 5);
			Log.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			Log.Text = "log";
			AddDate.AutoSize = false;
			AddDate.Size = new Size((int)Graphics.MeasureString("add date", AddDate.Font).Width + 24, 21);
			AddDate.Location = new Point(5, t += 5 + 21);
			AddDate.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			AddDate.Text = "add date";
			AddDate.CheckedChanged += AddDateChanged;
			AddDescription.AutoSize = false;
			AddDescription.Size = new Size((int)Graphics.MeasureString("add description", AddDescription.Font).Width + 24, 21);
			AddDescription.Location = new Point(5, t += 5 + 21);
			AddDescription.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			AddDescription.Text = "add description";
			AddDescription.CheckedChanged += AddDescriptionChanged;
			ImproveQuality.AutoSize = false;
			ImproveQuality.Size = new Size((int)Graphics.MeasureString("improve quality", ImproveQuality.Font).Width + 24, 21);
			ImproveQuality.Location = new Point(5, t += 5 + 21);
			ImproveQuality.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			ImproveQuality.Text = "improve quality";
			ImproveQuality.CheckedChanged += ImproveQualityChanged;
			PreferHevc.AutoSize = false;
			PreferHevc.Size = new Size((int)Graphics.MeasureString("prefer HEVC/H.265", PreferHevc.Font).Width + 24, 21);
			PreferHevc.Location = new Point(5, t += 5 + 21);
			PreferHevc.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			PreferHevc.Text = "prefer HEVC/H.265";
			PreferHevc.CheckedChanged += PreferHevcChanged;
			DirLabel.AutoSize = false;
			DirLabel.Size = new Size(w, 21);
			DirLabel.Location = new Point(5, t += 5 + 21);
			DirLabel.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			DirLabel.Text = "directory:";
			DirLabel.TextAlign = ContentAlignment.MiddleLeft;
			DirInput.Size = new Size(210 - 26, 21);
			DirInput.Location = new Point(2 * 5 + w, t);
			DirInput.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left);
			DirInput.Text = Dir;
			DirInput.Leave += DirLeave;
			OpenDir.Size = new Size(21, 21);
			OpenDir.Location = new Point(2 * 5 + w + 210 - 21, t);
			OpenDir.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right);
			OpenDir.Text = "⯇";
			OpenDir.Click += OpenDirClick;
			TypeLabel.AutoSize = false;
			TypeLabel.Size = new Size(w, 21);
			TypeLabel.Location = new Point(5, t += 5 + 21);
			TypeLabel.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			TypeLabel.Text = "type:";
			TypeLabel.TextAlign = ContentAlignment.MiddleLeft;
			TypeInput.Size = new Size(210, 21);
			TypeInput.Location = new Point(2 * 5 + w, t);
			TypeInput.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left);
			TypeInput.DropDownStyle = ComboBoxStyle.DropDownList;
			TypeInput.Items.Add("user videos");
			TypeInput.Items.Add("user favorites");
			TypeInput.Items.Add("music videos");
			TypeInput.Items.Add("hashtag videos");
			TypeInput.Items.Add("videos");
			TypeInput.SelectedIndex = 0;
			TypeInput.SelectedIndexChanged += TypeSelectedIndexChanged;
			FileLabel.AutoSize = false;
			FileLabel.Size = new Size(w, 21);
			FileLabel.Location = new Point(5, t += 5 + 21);
			FileLabel.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			FileLabel.Text = "file:";
			FileLabel.TextAlign = ContentAlignment.MiddleLeft;
			FileInput.Size = new Size(210 - 26, 21);
			FileInput.Location = new Point(2 * 5 + w, t);
			FileInput.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left);
			OpenFile.Size = new Size(21, 21);
			OpenFile.Location = new Point(2 * 5 + w + 210 - 21, t);
			OpenFile.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right);
			OpenFile.Text = "⯇";
			OpenFile.Click += OpenFileClick;
			LimitLabel.AutoSize = false;
			LimitLabel.Size = new Size(w, 21);
			LimitLabel.Location = new Point(5, t += 5 + 21);
			LimitLabel.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			LimitLabel.Text = "max:";
			LimitLabel.TextAlign = ContentAlignment.MiddleLeft;
			LimitInput.Size = new Size(210, 21);
			LimitInput.Location = new Point(2 * 5 + w, t);
			LimitInput.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left);
			w = 3 * 5 + w + 210;
			Download.Size = new Size(90, 21);
			Download.Location = new Point((w - 90) / 2, t += 5 + 21);
			Download.Anchor = AnchorStyles.None;
			Download.Text = "download";
			Download.Click += DownloadClick;
			Output.Size = new Size(w - 2 * 5, 210);
			Output.Location = new Point(5, t += 5 + 21);
			Output.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom | AnchorStyles.Left);
			Output.Multiline = true;
			Output.ReadOnly = true;
			Output.BackColor = Color.FromKnownColor(KnownColor.Window);
			Output.WordWrap = false;
			Output.ScrollBars = ScrollBars.Both;
			Tip.ShowAlways = true;
			Tip.AutoPopDelay = 9000;
			Tip.InitialDelay = 300;
			Tip.ReshowDelay = 300;
			Tip.SetToolTip(DirInput, "directory");
			Tip.SetToolTip(TypeInput, "type");
			Tip.SetToolTip(LimitInput, "max (optional - integer) - max number of videos to check");
			Tip.SetToolTip(FileInput, "file with responses of /aweme/v1/aweme/post/ (separated by whitespace)");
			Tip.SetToolTip(Output, "output");
			ClientSize = new Size(w, t + 5 + 210);
			Text = "fetch";
			ShowIcon = false;
			Controls.Add(Log);
			Controls.Add(AddDate);
			Controls.Add(AddDescription);
			Controls.Add(ImproveQuality);
			Controls.Add(PreferHevc);
			Controls.Add(DirLabel);
			Controls.Add(DirInput);
			Controls.Add(OpenDir);
			Controls.Add(TypeLabel);
			Controls.Add(TypeInput);
			Controls.Add(FileLabel);
			Controls.Add(FileInput);
			Controls.Add(OpenFile);
			Controls.Add(LimitLabel);
			Controls.Add(LimitInput);
			Controls.Add(Download);
			Controls.Add(Output);
			ResumeLayout(false);
			PerformLayout();
			Show();
			MinimumSize = Size;
			FileInput.Focus();
		}
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (Logger != null) {
					Logger.Close();
					Logger.Dispose();
				}
				if (Graphics != null) {
					Graphics.Dispose();
				}
				if (Components != null) {
					Components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
		public void DisableControls() {
			Stop = false;
			Download.Text = "stop";
			Log.Enabled = false;
			AddDate.Enabled = false;
			AddDescription.Enabled = false;
			ImproveQuality.Enabled = false;
			PreferHevc.Enabled = false;
			DirInput.ReadOnly = true;
			OpenDir.Enabled = false;
			TypeInput.Enabled = false;
			FileInput.ReadOnly = true;
			OpenFile.Enabled = false;
			LimitInput.ReadOnly = true;
			Cursor.Current = Cursors.WaitCursor;
		}
		public void EnableControls() {
			if (Logger != null) {
				Logger.Close();
				Logger.Dispose();
				Logger = null;
			}
			Stop = false;
			Download.Text = "download";
			Log.Enabled = true;
			AddDate.Enabled = true;
			AddDescription.Enabled = true;
			ImproveQuality.Enabled = true;
			PreferHevc.Enabled = true;
			DirInput.ReadOnly = false;
			OpenDir.Enabled = true;
			TypeInput.Enabled = true;
			FileInput.ReadOnly = false;
			OpenFile.Enabled = true;
			LimitInput.ReadOnly = false;
			Cursor.Current = Cursors.Default;
		}
		public static double Max(params double[] values) {
			if (values.Length == 0) {
				return 0;
			}
			int i = 0;
			double m = values[0];
			do {
				if (m < values[i]) {
					m = values[i];
				}
				i++;
			} while (i < values.Length);
			return m;
		}
		public static string GetFullDir(string directory, string root) {
			if (directory.Length == 0) {
				return root;
			}
			string t;
			switch (directory[directory.Length - 1]) {
				case '/':
				case '\\':
					if (directory.Length == 1) {
						return root;
					}
					t = directory.Substring(0, directory.Length - 1);
					break;
				default:
					t = directory;
					break;
			}
			return Path.GetFullPath(Path.IsPathRooted(t) ? t : root + Path.DirectorySeparatorChar + t);
		}
		public static int Substream(Stream source, int length, Stream destination, byte[] buffer) {
			if (length == 0) {
				return 0;
			}
			byte[] b = buffer != null ? buffer : new byte[length > 0 && length < 1024 ? length : 1024];
			int i, e = 0, j = b.Length;
			if (length > 0) {
				do {
					if (e + j > length) {
						j = length - e;
					}
					if ((i = source.Read(b, 0, j)) > 0) {
						destination.Write(b, 0, i);
						e += i;
					} else {
						break;
					}
				} while (true);
			} else {
				while ((i = source.Read(b, 0, j)) > 0) {
					destination.Write(b, 0, i);
					e += i;
				}
			}
			return e;
		}
		public static JsonIndex[] IndexJsonStream(Stream value) {
			int i = 0, e, j, b = value.ReadByte();
			List<JsonIndex> r = new List<JsonIndex>();
			while (b > -1) {
				do {
					switch ((char)b) {
						case ' ':
						case '\n':
						case '\r':
						case '\t':
							i++;
							b = value.ReadByte();
							break;
						default:
							goto w;
					}
				} while (b > -1);
				if (b < 0) {
					break;
				}
				w: switch((char)b) {
					case 'n':
						if ((char)(b = value.ReadByte()) != 'u' || (char)(b = value.ReadByte()) != 'l' || (char)(b = value.ReadByte()) != 'l') {
							return null;
						}
						r.Add(new JsonIndex {
							Type = 'n',
							Index = i,
							Length = 4
						});
						i += 4;
						b = value.ReadByte();
						break;
					case 'f':
						if ((char)(b = value.ReadByte()) != 'a' || (char)(b = value.ReadByte()) != 'l' || (char)(b = value.ReadByte()) != 's' || (char)(b = value.ReadByte()) != 'e') {
							return null;
						}
						r.Add(new JsonIndex {
							Type = 'f',
							Index = i,
							Length = 5
						});
						i += 5;
						b = value.ReadByte();
						break;
					case 't':
						if ((char)(b = value.ReadByte()) != 'r' || (char)(b = value.ReadByte()) != 'u' || (char)(b = value.ReadByte()) != 'e') {
							return null;
						}
						r.Add(new JsonIndex {
							Type = 't',
							Index = i,
							Length = 4
						});
						i += 4;
						b = value.ReadByte();
						break;
					case '-':
					case '0':
					case '1':
					case '2':
					case '3':
					case '4':
					case '5':
					case '6':
					case '7':
					case '8':
					case '9':
						e = i;
						i++;
						while ((b = value.ReadByte()) > -1) {
							switch ((char)b) {
								case '+':
								case '-':
								case '.':
								case '0':
								case '1':
								case '2':
								case '3':
								case '4':
								case '5':
								case '6':
								case '7':
								case '8':
								case '9':
								case 'E':
								case 'e':
									i++;
									break;
								default:
									goto n;
							}
						}
						n: r.Add(new JsonIndex {
							Type = '0',
							Index = e,
							Length = i - e
						});
						break;
					case '"':
						e = i;
						i++;
						while ((b = value.ReadByte()) > -1) {
							switch ((char)b) {
								case '\\':
									i += 2;
									if ((b = value.ReadByte()) < 0) {
										return null;
									}
									break;
								case '"':
									goto s;
								default:
									i++;
									break;
							}
						}
						if (b < 0) {
							return null;
						}
						s: i++;
						b = value.ReadByte();
						r.Add(new JsonIndex {
							Type = '"',
							Index = e,
							Length = i - e
						});
						break;
					case '[':
						e = i;
						j = 1;
						i++;
						while ((b = value.ReadByte()) > -1) {
							switch ((char)b) {
								case '"':
									i++;
									while ((b = value.ReadByte()) > -1) {
										switch ((char)b) {
											case '\\':
												i += 2;
												if ((b = value.ReadByte()) < 0) {
													return null;
												}
												break;
											case '"':
												goto sa;
											default:
												i++;
												break;
										}
									}
									if (b < 0) {
										return null;
									}
									sa: break;
								case '[':
									j++;
									break;
								case ']':
									j--;
									if (j == 0) {
										goto a;
									}
									break;
							}
							i++;
						}
						if (j != 0) {
							return null;
						}
						a: i++;
						b = value.ReadByte();
						r.Add(new JsonIndex {
							Type = '[',
							Index = e,
							Length = i - e
						});
						break;
					case '{':
						e = i;
						j = 1;
						i++;
						while ((b = value.ReadByte()) > -1) {
							switch ((char)b) {
								case '"':
									i++;
									while ((b = value.ReadByte()) > -1) {
										switch ((char)b) {
											case '\\':
												i += 2;
												if ((b = value.ReadByte()) < 0) {
													return null;
												}
												break;
											case '"':
												goto so;
											default:
												i++;
												break;
										}
									}
									if (b < 0) {
										return null;
									}
									so: break;
								case '{':
									j++;
									break;
								case '}':
									j--;
									if (j == 0) {
										goto o;
									}
									break;
							}
							i++;
						}
						if (j != 0) {
							return null;
						}
						o: i++;
						b = value.ReadByte();
						r.Add(new JsonIndex {
							Type = '{',
							Index = e,
							Length = i - e
						});
						break;
					default:
						return null;
				}
			}
			return r.ToArray();
		}
		public static string CreateTimeToString(long value) {
			return (new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).AddSeconds(value).ToString("yyyy-MM-dd HH':'mm':'ss");
		}
		public static string CreateTimeToFilename(long value) {
			return (new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).AddSeconds(value).ToString("yyyyMMddTHHmmssZ");
		}
		public static string CreateTimeToLog(long value) {
			return (new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).AddSeconds(value).ToString("yyyy-MM-ddTHH':'mm':'ssZ");
		}
		public static string SanitizeFilename(string value) {
			if (value.Length == 0) {
				return "";
			}
			int i = 0, e = 0;
			char[] r = new char[value.Length];
			do {
				switch (value[i]) {
					case '"':
					case '*':
					case '/':
					case ':':
					case '<':
					case '>':
					case '?':
					case '\\':
					case '|':
						break;
					default:
						if (value[i] >= ' ') {
							r[e] = value[i];
							e++;
						}
						break;
				}
				i++;
			} while (i < value.Length);
			return new string(r, 0, e);
		}
		public static string SanitizeLog(string value) {
			if (value.Length == 0) {
				return "";
			}
			int i = 0, e = 0;
			char[] r = new char[value.Length];
			do {
				switch (value[i]) {
					case '\r':
					case '\n':
						break;
					default:
						r[e] = value[i];
						e++;
						break;
				}
				i++;
			} while (i < value.Length);
			return new string(r, 0, e);
		}
		/*public static string GenerateBase62(int length) {
			const string base62 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
			int i = 0;
			char[] a = new char[length];
			Random r = new Random();
			while (i < length) {
				a[i] = base62[r.Next(62)];
				i++;
			}
			return new string(a);
		}*/
		/*public static string GenerateDeviceId() {
			int i = 1;
			Random r = new Random();
			char[] a = new char[19];//11 + r.Next(9)
			a[0] = '6';//(char)(49 + r.Next(9))
			while (i < a.Length) {
				a[i] = (char)(48 + r.Next(10));
				i++;
			}
			return new string(a);
		}*/
		/*public static string GeneratePublicIp() {
			int i0, i1, i2;
			Random r = new Random();
			while ((i0 = r.Next(1, 224)) == 10 || i0 == 127) {}
			switch (i0) {
				case 100:
					while ((i1 = r.Next(256)) > 63 && i1 < 128) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 169:
					while ((i1 = r.Next(256)) == 254) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 172:
					while ((i1 = r.Next(256)) > 15 && i1 < 32) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 192:
					while ((i1 = r.Next(256)) == 168) {}
					switch (i1) {
						case 0:
							while ((i2 = r.Next(1, 256)) == 2) {}
							return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
						case 88:
							while ((i2 = r.Next(256)) == 99) {}
							return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 198:
					while ((i1 = r.Next(256)) == 18 || i1 == 19) {}
					if (i1 == 51) {
						while ((i2 = r.Next(256)) == 100) {}
						return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 203:
					if ((i1 = r.Next(256)) == 0) {
						while ((i2 = r.Next(256)) == 113) {}
						return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
			}
			return i0.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
		}*/
		/*public static string GenerateIp() {
			Random r = new Random();
			return r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString();
		}*/
		public static string GetQualityUri(string value) {
			int i = value.IndexOf("?video_id="), e;
			if (i > -1 && (e = value.IndexOf("&", i + 10)) > -1) {
				return "https://api.tiktokv.com/aweme/v1/play/" + value.Substring(i, e - i) + "&improve_bitrate=1&ratio=default";
			}
			return value;
		}
		public string[] GetVideoUrlList(VideoData value) {
			if (Hevc) {
				if (value.PlayAddr265 != null && value.PlayAddr265.UrlList != null && value.PlayAddr265.UrlList.Length > 0) {
					return value.PlayAddr265.UrlList;
				} else if (value.PlayAddrBytevc1 != null && value.PlayAddrBytevc1.UrlList != null && value.PlayAddrBytevc1.UrlList.Length > 0) {
					return value.PlayAddrBytevc1.UrlList;
				}
			}
			if (value.PlayAddr != null && value.PlayAddr.UrlList != null && value.PlayAddr.UrlList.Length > 0) {
				return value.PlayAddr.UrlList;
			}
			return null;
		}
		public void DownloadAweme(object aweme) {
			Aweme a = (Aweme)aweme;
			try {
				bool f = false;
				int r = 0;
				string n, d = a.AddDate && a.Date > 0 ? CreateTimeToFilename(a.Date) : "", t = a.AddDescription && a.Description.Length > 0 ? SanitizeFilename(a.Description) : "";
				byte[] l;
				if (File.Exists(n = (d.Length > 0 ? a.Id + " " + d : a.Id) + (t.Length > 0 ? " " + t + ".mp4" : ".mp4"))) {
					return;
				}
				AwemeWebClient c = new AwemeWebClient();
				do {
					try {
						c.Headers.Set("User-Agent", "okhttp");
						c.DownloadFile(a.Url, n);
						if ((new FileInfo(n)).Length == 0) {
							File.Delete(n);
							throw new Exception("File is empty.");
						}
						break;
					} catch (PathTooLongException) {
						if (t.Length == 0 || f) {
							throw;
						} else if (File.Exists(n = (d.Length > 0 ? a.Id + " " + d : a.Id) + ".mp4")) {
							return;
						}
						f = true;
						r = 0;
					} catch (Exception) {
						if (r == 3) {
							File.Delete(n);
							throw;
						}
						r++;
					}
				} while (true);
				lock (OutLock) {
					Output.AppendText(a.Id);
					Output.AppendText(" is downloaded");
					Output.AppendText(Environment.NewLine);
					if (Logger != null) {
						l = Encoding.UTF8.GetBytes(a.Id + " " + CreateTimeToLog(a.Date) + " " + SanitizeLog(a.Description) + Environment.NewLine);
						Logger.Write(l, 0, l.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(a.Id);
					Output.AppendText(" failed");
					Output.AppendText(Environment.NewLine);
					Output.AppendText(x.InnerException != null ? x.InnerException.Message : x.Message);
					Output.AppendText(Environment.NewLine);
				}
			} finally {
				lock (SyncLock) {
					Count--;
					Monitor.Pulse(SyncLock);
				}
			}
		}
		public void DownloadUserVideos(object download) {
			try {
				Download d = (Download)download;
				long i = 0, e = 0, j = 0;
				byte[] b = new byte[1024];
				string[] u;
				JsonIndex[] ji;
				MemoryStream m, t;
				FileStream f;
				DataContractJsonSerializer js = new DataContractJsonSerializer(typeof(UserVideosData));
				UserVideosData o;
				using (m = new MemoryStream()) {
					using (f = new FileStream(Path.IsPathRooted(d.Filename) ? d.Filename : Root + Path.DirectorySeparatorChar + d.Filename, FileMode.Open, FileAccess.Read, FileShare.Read)) {
						Substream(f, -1, m, b);
					}
					m.Seek(0, SeekOrigin.Begin);
					ji = IndexJsonStream(m);
					if (ji == null || ji.Length == 0) {
						goto end;
					}
					using (t = new MemoryStream()) {
						do {
							m.Seek(ji[j].Index, SeekOrigin.Begin);
							t.Seek(0, SeekOrigin.Begin);
							t.SetLength(Substream(m, ji[j].Length, t, b));
							t.Seek(0, SeekOrigin.Begin);
							o = (UserVideosData)js.ReadObject(t);
							if (o != null && o.AwemeList != null) {
								for (i = 0; i < o.AwemeList.Length; i++, e++) {
									if (e == d.Limit) {
										j = ji.Length - 1;
										break;
									} else if (o.AwemeList[i].Video != null && (u = GetVideoUrlList(o.AwemeList[i].Video)) != null) {
										lock (SyncLock) {
											Count++;
										}
										new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
											Id = o.AwemeList[i].AwemeId,
											Date = o.AwemeList[i].CreateTime,
											Description = o.AwemeList[i].Desc != null ? o.AwemeList[i].Desc : "",
											Url = Quality ? GetQualityUri(u[u.Length - 1]) : u[0],
											AddDate = Date,
											AddDescription = Description
										});
										lock (SyncLock) {
											if (Count >= 10) {
												Monitor.Wait(SyncLock);
											}
										}
									}
								}
							} else {
								lock (StopLock) {
									if (Stop) {
										break;
									}
								}
							}
							j++;
						} while (j < ji.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			end: EnableControls();
		}
		public void DownloadUserFavorites(object download) {
			try {
				Download d = (Download)download;
				long i = 0, e = 0, j = 0;
				byte[] b = new byte[1024];
				string[] u;
				JsonIndex[] ji;
				MemoryStream m, t;
				FileStream f;
				DataContractJsonSerializer js = new DataContractJsonSerializer(typeof(UserVideosData));
				UserVideosData o;
				using (m = new MemoryStream()) {
					using (f = new FileStream(Path.IsPathRooted(d.Filename) ? d.Filename : Root + Path.DirectorySeparatorChar + d.Filename, FileMode.Open, FileAccess.Read, FileShare.Read)) {
						Substream(f, -1, m, b);
					}
					m.Seek(0, SeekOrigin.Begin);
					ji = IndexJsonStream(m);
					if (ji == null || ji.Length == 0) {
						goto end;
					}
					using (t = new MemoryStream()) {
						do {
							m.Seek(ji[j].Index, SeekOrigin.Begin);
							t.Seek(0, SeekOrigin.Begin);
							t.SetLength(Substream(m, ji[j].Length, t, b));
							t.Seek(0, SeekOrigin.Begin);
							o = (UserVideosData)js.ReadObject(t);
							if (o != null && o.AwemeList != null) {
								for (i = 0; i < o.AwemeList.Length; i++, e++) {
									if (e == d.Limit) {
										j = ji.Length - 1;
										break;
									} else if (o.AwemeList[i].Video != null && (u = GetVideoUrlList(o.AwemeList[i].Video)) != null) {
										lock (SyncLock) {
											Count++;
										}
										new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
											Id = o.AwemeList[i].AwemeId,
											Date = o.AwemeList[i].CreateTime,
											Description = o.AwemeList[i].Desc != null ? o.AwemeList[i].Desc : "",
											Url = Quality ? GetQualityUri(u[u.Length - 1]) : u[0],
											AddDate = Date,
											AddDescription = Description
										});
										lock (SyncLock) {
											if (Count >= 10) {
												Monitor.Wait(SyncLock);
											}
										}
									}
								}
							} else {
								lock (StopLock) {
									if (Stop) {
										break;
									}
								}
							}
							j++;
						} while (j < ji.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			end: EnableControls();
		}
		public void DownloadMusicVideos(object download) {
			try {
				Download d = (Download)download;
				long i = 0, e = 0, j = 0;
				byte[] b = new byte[1024];
				string[] u;
				JsonIndex[] ji;
				MemoryStream m, t;
				FileStream f;
				DataContractJsonSerializer js = new DataContractJsonSerializer(typeof(MusicVideosData));
				MusicVideosData o;
				using (m = new MemoryStream()) {
					using (f = new FileStream(Path.IsPathRooted(d.Filename) ? d.Filename : Root + Path.DirectorySeparatorChar + d.Filename, FileMode.Open, FileAccess.Read, FileShare.Read)) {
						Substream(f, -1, m, b);
					}
					m.Seek(0, SeekOrigin.Begin);
					ji = IndexJsonStream(m);
					if (ji == null || ji.Length == 0) {
						goto end;
					}
					using (t = new MemoryStream()) {
						do {
							m.Seek(ji[j].Index, SeekOrigin.Begin);
							t.Seek(0, SeekOrigin.Begin);
							t.SetLength(Substream(m, ji[j].Length, t, b));
							t.Seek(0, SeekOrigin.Begin);
							o = (MusicVideosData)js.ReadObject(t);
							if (o != null && o.AwemeList != null) {
								for (i = 0; i < o.AwemeList.Length; i++, e++) {
									if (e == d.Limit) {
										j = ji.Length - 1;
										break;
									} else if (o.AwemeList[i].Video != null && (u = GetVideoUrlList(o.AwemeList[i].Video)) != null) {
										lock (SyncLock) {
											Count++;
										}
										new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
											Id = o.AwemeList[i].AwemeId,
											Date = o.AwemeList[i].CreateTime,
											Description = o.AwemeList[i].Desc != null ? o.AwemeList[i].Desc : "",
											Url = Quality ? GetQualityUri(u[u.Length - 1]) : u[0],
											AddDate = Date,
											AddDescription = Description
										});
										lock (SyncLock) {
											if (Count >= 10) {
												Monitor.Wait(SyncLock);
											}
										}
									}
								}
							} else {
								lock (StopLock) {
									if (Stop) {
										break;
									}
								}
							}
							j++;
						} while (j < ji.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			end: EnableControls();
		}
		public void DownloadHashtagVideos(object download) {
			try {
				Download d = (Download)download;
				long i = 0, e = 0, j = 0;
				byte[] b = new byte[1024];
				string[] u;
				JsonIndex[] ji;
				MemoryStream m, t;
				FileStream f;
				DataContractJsonSerializer js = new DataContractJsonSerializer(typeof(HashtagVideosData));
				HashtagVideosData o;
				using (m = new MemoryStream()) {
					using (f = new FileStream(Path.IsPathRooted(d.Filename) ? d.Filename : Root + Path.DirectorySeparatorChar + d.Filename, FileMode.Open, FileAccess.Read, FileShare.Read)) {
						Substream(f, -1, m, b);
					}
					m.Seek(0, SeekOrigin.Begin);
					ji = IndexJsonStream(m);
					if (ji == null || ji.Length == 0) {
						goto end;
					}
					using (t = new MemoryStream()) {
						do {
							m.Seek(ji[j].Index, SeekOrigin.Begin);
							t.Seek(0, SeekOrigin.Begin);
							t.SetLength(Substream(m, ji[j].Length, t, b));
							t.Seek(0, SeekOrigin.Begin);
							o = (HashtagVideosData)js.ReadObject(t);
							if (o != null && o.AwemeList != null) {
								for (i = 0; i < o.AwemeList.Length; i++, e++) {
									if (e == d.Limit) {
										j = ji.Length - 1;
										break;
									} else if (o.AwemeList[i].Video != null && (u = GetVideoUrlList(o.AwemeList[i].Video)) != null) {
										lock (SyncLock) {
											Count++;
										}
										new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
											Id = o.AwemeList[i].AwemeId,
											Date = o.AwemeList[i].CreateTime,
											Description = o.AwemeList[i].Desc != null ? o.AwemeList[i].Desc : "",
											Url = Quality ? GetQualityUri(u[u.Length - 1]) : u[0],
											AddDate = Date,
											AddDescription = Description
										});
										lock (SyncLock) {
											if (Count >= 10) {
												Monitor.Wait(SyncLock);
											}
										}
									}
								}
							} else {
								lock (StopLock) {
									if (Stop) {
										break;
									}
								}
							}
							j++;
						} while (j < ji.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			end: EnableControls();
		}
		public void DownloadVideos(object download) {
			try {
				Download d = (Download)download;
				long i = 0, e = 0, j = 0;
				byte[] b = new byte[1024];
				string[] u;
				JsonIndex[] ji;
				MemoryStream m, t;
				FileStream f;
				DataContractJsonSerializer js = new DataContractJsonSerializer(typeof(VideosData));
				VideosData o;
				using (m = new MemoryStream()) {
					using (f = new FileStream(Path.IsPathRooted(d.Filename) ? d.Filename : Root + Path.DirectorySeparatorChar + d.Filename, FileMode.Open, FileAccess.Read, FileShare.Read)) {
						Substream(f, -1, m, b);
					}
					m.Seek(0, SeekOrigin.Begin);
					ji = IndexJsonStream(m);
					if (ji == null || ji.Length == 0) {
						goto end;
					}
					using (t = new MemoryStream()) {
						do {
							m.Seek(ji[j].Index, SeekOrigin.Begin);
							t.Seek(0, SeekOrigin.Begin);
							t.SetLength(Substream(m, ji[j].Length, t, b));
							t.Seek(0, SeekOrigin.Begin);
							o = (VideosData)js.ReadObject(t);
							if (o != null && o.AwemeDetails != null) {
								for (i = 0; i < o.AwemeDetails.Length; i++, e++) {
									if (e == d.Limit) {
										j = ji.Length - 1;
										break;
									} else if (o.AwemeDetails[i].Video != null && (u = GetVideoUrlList(o.AwemeDetails[i].Video)) != null) {
										lock (SyncLock) {
											Count++;
										}
										new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
											Id = o.AwemeDetails[i].AwemeId,
											Date = o.AwemeDetails[i].CreateTime,
											Description = o.AwemeDetails[i].Desc != null ? o.AwemeDetails[i].Desc : "",
											Url = Quality ? GetQualityUri(u[u.Length - 1]) : u[0],
											AddDate = Date,
											AddDescription = Description
										});
										lock (SyncLock) {
											if (Count >= 10) {
												Monitor.Wait(SyncLock);
											}
										}
									}
								}
							} else {
								lock (StopLock) {
									if (Stop) {
										break;
									}
								}
							}
							j++;
						} while (j < ji.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			end: EnableControls();
		}
		public void AddDateChanged(object sender, EventArgs args) {
			Date = AddDate.Checked;
		}
		public void AddDescriptionChanged(object sender, EventArgs args) {
			Description = AddDescription.Checked;
		}
		public void ImproveQualityChanged(object sender, EventArgs args) {
			Quality = ImproveQuality.Checked;
		}
		public void PreferHevcChanged(object sender, EventArgs args) {
			Hevc = PreferHevc.Checked;
		}
		public void DirLeave(object sender, EventArgs args) {
			try {
				string d0 = GetFullDir(Dir, Root), d1 = GetFullDir(DirInput.Text, Root);
				if (d0 != d1) {
					if (!Directory.Exists(d1)) {
						Directory.CreateDirectory(d1);
					}
					Directory.SetCurrentDirectory(d1);
					Dir = DirInput.Text;
				}
			} catch (Exception x) {
				MessageBox.Show(x.Message);
				DirInput.Text = Dir;
			}
		}
		public void OpenDirClick(object sender, EventArgs args) {
			if (DirOpener.ShowDialog() != DialogResult.OK) {
				return;
			}
			DirInput.Text = DirOpener.SelectedPath;
			DirLeave(DirInput, null);
		}
		public void TypeSelectedIndexChanged(object sender, EventArgs args) {
			switch (TypeInput.SelectedIndex) {
				case 0:
					Tip.SetToolTip(FileInput, "file with responses of /aweme/v1/aweme/post/ (separated by whitespace)");
					break;
				case 1:
					Tip.SetToolTip(FileInput, "file with responses of /aweme/v1/aweme/favorite/ (separated by whitespace)");
					break;
				case 2:
					Tip.SetToolTip(FileInput, "file with responses of /aweme/v1/music/aweme/ (separated by whitespace)");
					break;
				case 3:
					Tip.SetToolTip(FileInput, "file with responses of /aweme/v1/challenge/aweme/ (separated by whitespace)");
					break;
				default://4
					Tip.SetToolTip(FileInput, "file with responses of /aweme/v1/multi/aweme/detail/ (separated by whitespace)");
					break;
			}
		}
		public void OpenFileClick(object sender, EventArgs args) {
			if (FileOpener.ShowDialog() != DialogResult.OK) {
				return;
			}
			FileInput.Text = FileOpener.FileName;
		}
		public void DownloadClick(object sender, EventArgs args) {
			lock (StopLock) {
				Stop = true;
				if (FileInput.ReadOnly) {
					return;
				}
			}
			DisableControls();
			if (Log.Checked) {
				try {
					Logger = new FileStream("log.txt", FileMode.Append);
				} catch (Exception x) {
					MessageBox.Show(x.Message);
					goto end;
				}
			}
			long l;
			if (FileInput.Text.Length == 0) {
				MessageBox.Show("File is empty.");
				goto end;
			} else if (LimitInput.Text.Length == 0) {
				l = -1;
			} else if (!Int64.TryParse(LimitInput.Text, NumberStyles.None, CultureInfo.InvariantCulture, out l)) {
				MessageBox.Show("Max is not valid.");
				goto end;
			}
			Output.Text = "";
			switch (TypeInput.SelectedIndex) {
				case 0:
					new Thread(new ParameterizedThreadStart(DownloadUserVideos)).Start(new Download {
						Filename =  FileInput.Text,
						Limit = l
					});
					break;
				case 1:
					new Thread(new ParameterizedThreadStart(DownloadUserFavorites)).Start(new Download {
						Filename =  FileInput.Text,
						Limit = l
					});
					return;
				case 2:
					new Thread(new ParameterizedThreadStart(DownloadMusicVideos)).Start(new Download {
						Filename =  FileInput.Text,
						Limit = l
					});
					return;
				case 3:
					new Thread(new ParameterizedThreadStart(DownloadHashtagVideos)).Start(new Download {
						Filename =  FileInput.Text,
						Limit = l
					});
					return;
				default:
					new Thread(new ParameterizedThreadStart(DownloadVideos)).Start(new Download {
						Filename =  FileInput.Text,
						Limit = l
					});
					return;
			}
			end: EnableControls();
		}
		/*public static bool ValidateRemoteCertificate(object sender, object certificate, object chain, SslPolicyErrors sslPolicyErrors) {
			return true;
		}*/
		[STAThread]
		public static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			ServicePointManager.SecurityProtocol |= SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12 | SecurityProtocolType.Tls13;//.NET 4.8
			//ServicePointManager.ServerCertificateValidationCallback += ValidateRemoteCertificate;//.NET 3.5
			string d = Path.GetDirectoryName(Environment.GetCommandLineArgs()[0]);
			d += d.Length > 0 ? Path.DirectorySeparatorChar + "download" : "download";
			try {
				if (!Directory.Exists(d)) {
					Directory.CreateDirectory(d);
				}
				Directory.SetCurrentDirectory(d);
			} catch {
				MessageBox.Show("Could not initialize download directory.");
				return;
			}
			Application.Run(new MainForm(Path.GetDirectoryName(d), Path.GetFileName(d)));
		}
	}
}