//csc.exe /target:winexe /optimize ids.cs
//.NET 4.8
using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Text;
using System.Collections.Generic;
using System.Net;
//using System.Net.Security;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Threading;

namespace AwemerApp {
	[DataContract]
	public class UrlData {
		[DataMember(Name = "url_list")]
		public string[] UrlList;
	}
	[DataContract]
	public class VideoData {
		[DataMember(Name = "play_addr")]
		public UrlData PlayAddr;
		//[DataMember(Name = "play_addr_264")]
		//public UrlData PlayAddr264;
		[DataMember(Name = "play_addr_265")]
		public UrlData PlayAddr265;
		[DataMember(Name = "play_addr_bytevc1")]
		public UrlData PlayAddrBytevc1;
	}
	[DataContract]
	public class AwemeDetailData {
		[DataMember(Name = "aweme_id")]
		public string AwemeId;
		[DataMember(Name = "create_time")]
		public long CreateTime;
		[DataMember(Name = "desc")]
		public string Desc;
		[DataMember(Name = "video")]
		public VideoData Video;
	}
	[DataContract]
	public class AwemesData {
		//[DataMember(Name = "status_code")]
		//public long StatusCode;
		[DataMember(Name = "aweme_details")]
		public AwemeDetailData[] AwemeDetails;
	}
	public struct Aweme {
		public string Id;
		public long Date;
		public string Description;
		public string Url;
		public bool AddDate;
		public bool AddDescription;
	}
	/*public class UnexpectedException: Exception {
		public UnexpectedException() {}
		public UnexpectedException(string message): base(message) {}
		public UnexpectedException(string message, Exception inner): base(message, inner) {}
	}*/
	public class AwemeWebClient: WebClient {
		protected override WebResponse GetWebResponse(WebRequest request) {
			(request as HttpWebRequest).AllowAutoRedirect = true;
			HttpWebResponse r = base.GetWebResponse(request) as HttpWebResponse;
			if (r.ContentLength == 0) {
				throw new Exception("Response body is empty.");
			}
			return r;
		}
		/*public bool DownloadNonEmptyFile(Uri uri, string filename, byte[] buffer) {
			int i;
			byte[] b;
			Stream s;
			FileStream f;
			using (s = OpenRead(uri)) {
				if ((i = s.ReadByte()) == -1) {
					return false;
				}
				b = buffer != null ? buffer : new byte[1024];
				using (f = new FileStream(filename, FileMode.Create)) {
					f.WriteByte((byte)i);
					while ((i = s.Read(b, 0, b.Length)) > 0) {
						f.Write(b, 0, i);
					}
					f.Flush();
				}
			}
			return true;
		}
		public bool DownloadNonEmptyFile(string uri, string filename, byte[] buffer) {
			return DownloadNonEmptyFile(new Uri(uri), filename, buffer);
		}*/
	}
	public class MainForm: Form {
		public string Root;
		public string Dir;
		public bool Stop;
		public bool Date;
		public bool Description;
		public bool Quality;
		public bool Hevc;
		public string DeviceId = GenerateDeviceId();//6827152794274924038
		public string IId = GenerateDeviceId();//6831237747954714373
		public string DeviceVersion = GenerateBase62(8);
		//public string Os = GenerateBase62(16);
		//public string Ip = "";
		public int Count;
		public FileStream Logger = null;
		public Graphics Graphics = null;
		public IContainer Components = new Container();
		public ToolTip Tip = new ToolTip();
		public FolderBrowserDialog DirOpener = new FolderBrowserDialog();
		public CheckBox Log = new CheckBox();
		public CheckBox AddDate = new CheckBox();
		public CheckBox AddDescription = new CheckBox();
		public CheckBox ImproveQuality = new CheckBox();
		public CheckBox PreferHevc = new CheckBox();
		public Label DirLabel = new Label();
		public TextBox DirInput = new TextBox();
		public Button OpenDir = new Button();
		public TextBox Input = new TextBox();
		public Button Download = new Button();
		public TextBox Output = new TextBox();
		public readonly object StopLock = new Object();
		public readonly object OutLock = new Object();
		public readonly object SyncLock = new Object();
		public MainForm(string root, string directory) {
			Root = root;
			Dir = directory;
			SuspendLayout();
			Components.Add(Tip);
			Components.Add(DirOpener);
			Graphics = CreateGraphics();
			int t, w = (int)Graphics.MeasureString("directory:", DirLabel.Font).Width + 5;
			DirOpener.RootFolder = Environment.SpecialFolder.Desktop;
			DirOpener.SelectedPath = Root + Path.DirectorySeparatorChar + Dir;
			Log.AutoSize = false;
			Log.Size = new Size((int)Graphics.MeasureString("log", Log.Font).Width + 24, 21);
			Log.Location = new Point(5, t = 5);
			Log.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			Log.Text = "log";
			AddDate.AutoSize = false;
			AddDate.Size = new Size((int)Graphics.MeasureString("add date", AddDate.Font).Width + 24, 21);
			AddDate.Location = new Point(5, t += 5 + 21);
			AddDate.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			AddDate.Text = "add date";
			AddDate.CheckedChanged += AddDateChanged;
			AddDescription.AutoSize = false;
			AddDescription.Size = new Size((int)Graphics.MeasureString("add description", AddDescription.Font).Width + 24, 21);
			AddDescription.Location = new Point(5, t += 5 + 21);
			AddDescription.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			AddDescription.Text = "add description";
			AddDescription.CheckedChanged += AddDescriptionChanged;
			ImproveQuality.AutoSize = false;
			ImproveQuality.Size = new Size((int)Graphics.MeasureString("improve quality", ImproveQuality.Font).Width + 24, 21);
			ImproveQuality.Location = new Point(5, t += 5 + 21);
			ImproveQuality.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			ImproveQuality.Text = "improve quality";
			ImproveQuality.CheckedChanged += ImproveQualityChanged;
			PreferHevc.AutoSize = false;
			PreferHevc.Size = new Size((int)Graphics.MeasureString("prefer HEVC/H.265", PreferHevc.Font).Width + 24, 21);
			PreferHevc.Location = new Point(5, t += 5 + 21);
			PreferHevc.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			PreferHevc.Text = "prefer HEVC/H.265";
			PreferHevc.CheckedChanged += PreferHevcChanged;
			DirLabel.AutoSize = false;
			DirLabel.Size = new Size(w, 21);
			DirLabel.Location = new Point(5, t += 5 + 21);
			DirLabel.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Left);
			DirLabel.Text = "directory:";
			DirLabel.TextAlign = ContentAlignment.MiddleLeft;
			DirInput.Size = new Size(210 - 26, 21);
			DirInput.Location = new Point(2 * 5 + w, t);
			DirInput.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left);
			DirInput.Text = Dir;
			DirInput.Leave += DirLeave;
			OpenDir.Size = new Size(21, 21);
			OpenDir.Location = new Point(2 * 5 + w + 210 - 21, t);
			OpenDir.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right);
			OpenDir.Text = "⯇";
			OpenDir.Click += OpenDirClick;
			w = 3 * 5 + w + 210;
			Input.Size = new Size(w - 2 * 5, 150);
			Input.Location = new Point(0, 0);
			Input.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom | AnchorStyles.Left);
			Input.Multiline = true;
			Input.WordWrap = false;
			Input.ScrollBars = ScrollBars.Both;
			Download.Size = new Size(90, 21);
			Download.Location = new Point((w - 90) / 2, 5);
			Download.Anchor = AnchorStyles.None;
			Download.Text = "download";
			Download.Click += DownloadClick;
			Output.Size = new Size(w - 2 * 5, 150);
			Output.Location = new Point(0, 0);
			Output.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom | AnchorStyles.Left);
			Output.Multiline = true;
			Output.ReadOnly = true;
			Output.BackColor = Color.FromKnownColor(KnownColor.Window);
			Output.WordWrap = false;
			Output.ScrollBars = ScrollBars.Both;
			TableLayoutPanel l = new TableLayoutPanel();
			l.Size = new Size(w - 2 * 5, 2 * 5 + 21 + 2 * 150);
			l.Location = new Point(5, t += 5 + 21);
			l.BorderStyle = BorderStyle.None;
			l.Anchor = (AnchorStyles)(AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom | AnchorStyles.Left);
			l.ColumnCount = 1;
			l.RowCount = 3;
			l.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, l.Width));
			l.RowStyles.Add(new RowStyle(SizeType.Percent, 50));
			l.RowStyles.Add(new RowStyle(SizeType.Absolute, 2 * 5 + 21));
			l.RowStyles.Add(new RowStyle(SizeType.Percent, 50));
			l.Controls.Add(Input, 0, 0);
			l.Controls.Add(Download, 0, 1);
			l.Controls.Add(Output, 0, 2);
			Tip.ShowAlways = true;
			Tip.AutoPopDelay = 9000;
			Tip.InitialDelay = 300;
			Tip.ReshowDelay = 300;
			Tip.SetToolTip(DirInput, "directory");
			Tip.SetToolTip(Input, "ids or urls of videos (separated by whitespace)");
			Tip.SetToolTip(Output, "output");
			ClientSize = new Size(w, t + 5 + l.Height);
			Text = "ids";
			ShowIcon = false;
			Controls.Add(Log);
			Controls.Add(AddDate);
			Controls.Add(AddDescription);
			Controls.Add(ImproveQuality);
			Controls.Add(PreferHevc);
			Controls.Add(DirLabel);
			Controls.Add(DirInput);
			Controls.Add(OpenDir);
			Controls.Add(l);
			ResumeLayout(false);
			PerformLayout();
			Show();
			MinimumSize = Size;
			Input.Focus();
		}
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (Logger != null) {
					Logger.Close();
					Logger.Dispose();
				}
				if (Graphics != null) {
					Graphics.Dispose();
				}
				if (Components != null) {
					Components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
		public void DisableControls() {
			Stop = false;
			Download.Text = "stop";
			Log.Enabled = false;
			AddDate.Enabled = false;
			AddDescription.Enabled = false;
			ImproveQuality.Enabled = false;
			PreferHevc.Enabled = false;
			DirInput.ReadOnly = true;
			OpenDir.Enabled = false;
			Input.ReadOnly = true;
			Cursor.Current = Cursors.WaitCursor;
		}
		public void EnableControls() {
			if (Logger != null) {
				Logger.Close();
				Logger.Dispose();
				Logger = null;
			}
			Stop = false;
			Download.Text = "download";
			Log.Enabled = true;
			AddDate.Enabled = true;
			AddDescription.Enabled = true;
			ImproveQuality.Enabled = true;
			PreferHevc.Enabled = true;
			DirInput.ReadOnly = false;
			OpenDir.Enabled = true;
			Input.ReadOnly = false;
			Cursor.Current = Cursors.Default;
		}
		public static string GetFullDir(string directory, string root) {
			if (directory.Length == 0) {
				return root;
			}
			string t;
			switch (directory[directory.Length - 1]) {
				case '/':
				case '\\':
					if (directory.Length == 1) {
						return root;
					}
					t = directory.Substring(0, directory.Length - 1);
					break;
				default:
					t = directory;
					break;
			}
			return Path.GetFullPath(Path.IsPathRooted(t) ? t : root + Path.DirectorySeparatorChar + t);
		}
		public static string CreateTimeToString(long value) {
			return (new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).AddSeconds(value).ToString("yyyy-MM-dd HH':'mm':'ss");
		}
		public static string CreateTimeToFilename(long value) {
			return (new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).AddSeconds(value).ToString("yyyyMMddTHHmmssZ");
		}
		public static string CreateTimeToLog(long value) {
			return (new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).AddSeconds(value).ToString("yyyy-MM-ddTHH':'mm':'ssZ");
		}
		public static string SanitizeFilename(string value) {
			if (value.Length == 0) {
				return "";
			}
			int i = 0, e = 0;
			char[] r = new char[value.Length];
			do {
				switch (value[i]) {
					case '"':
					case '*':
					case '/':
					case ':':
					case '<':
					case '>':
					case '?':
					case '\\':
					case '|':
						break;
					default:
						if (value[i] >= ' ') {
							r[e] = value[i];
							e++;
						}
						break;
				}
				i++;
			} while (i < value.Length);
			return new string(r, 0, e);
		}
		public static string SanitizeLog(string value) {
			if (value.Length == 0) {
				return "";
			}
			int i = 0, e = 0;
			char[] r = new char[value.Length];
			do {
				switch (value[i]) {
					case '\r':
					case '\n':
						break;
					default:
						r[e] = value[i];
						e++;
						break;
				}
				i++;
			} while (i < value.Length);
			return new string(r, 0, e);
		}
		public static string GenerateBase62(int length) {
			const string base62 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
			int i = 0;
			char[] a = new char[length];
			Random r = new Random();
			while (i < length) {
				a[i] = base62[r.Next(62)];
				i++;
			}
			return new string(a);
		}
		public static string GenerateDeviceId() {
			int i = 1;
			Random r = new Random();
			char[] a = new char[19];//11 + r.Next(9)
			a[0] = '6';//(char)(49 + r.Next(9))
			while (i < a.Length) {
				a[i] = (char)(48 + r.Next(10));
				i++;
			}
			return new string(a);
		}
		/*public static string GeneratePublicIp() {
			int i0, i1, i2;
			Random r = new Random();
			while ((i0 = r.Next(1, 224)) == 10 || i0 == 127) {}
			switch (i0) {
				case 100:
					while ((i1 = r.Next(256)) > 63 && i1 < 128) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 169:
					while ((i1 = r.Next(256)) == 254) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 172:
					while ((i1 = r.Next(256)) > 15 && i1 < 32) {}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 192:
					while ((i1 = r.Next(256)) == 168) {}
					switch (i1) {
						case 0:
							while ((i2 = r.Next(1, 256)) == 2) {}
							return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
						case 88:
							while ((i2 = r.Next(256)) == 99) {}
							return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 198:
					while ((i1 = r.Next(256)) == 18 || i1 == 19) {}
					if (i1 == 51) {
						while ((i2 = r.Next(256)) == 100) {}
						return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
				case 203:
					if ((i1 = r.Next(256)) == 0) {
						while ((i2 = r.Next(256)) == 113) {}
						return i0.ToString() + "." + i1.ToString() + "." + i2.ToString() + "." + r.Next(256).ToString();
					}
					return i0.ToString() + "." + i1.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
			}
			return i0.ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString() + "." + r.Next(256).ToString();
		}*/
		/*public static string GenerateIp() {
			Random r = new Random();
			return r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString() + '.' + r.Next(0, 256).ToString();
		}*/
		public static string GetQualityUri(string value) {
			int i = value.IndexOf("?video_id="), e;
			if (i > -1 && (e = value.IndexOf("&", i + 10)) > -1) {
				return "https://api.tiktokv.com/aweme/v1/play/" + value.Substring(i, e - i) + "&improve_bitrate=1&ratio=default";
			}
			return value;
		}
		public static string GetRedirect(string url) {
			int i = 0;
			string r = url;
			HttpWebRequest rq;
			HttpWebResponse rs;
			do {
				rq = (HttpWebRequest)HttpWebRequest.Create(url);
				rq.Method = "GET";
				rq.AllowAutoRedirect = false;
				rq.UserAgent = "Mozilla/5.0 AppleWebKit/537.36 Chrome/87.0.4280.88 Safari/537.36";
				using (rs = (HttpWebResponse)rq.GetResponse()) {
					if ((int)rs.StatusCode < 300 || (int)rs.StatusCode >= 400) {
						break;
					} else if ((r = rs.Headers.Get("Location")) == null) {
						r = url;
						break;
					} else if (r.IndexOf("://") == -1) {
						r = (new Uri(new Uri(url), r)).ToString();
					}
				}
				url = r;
				i++;
			} while (i < 9);
			return r;
		}
		public static string ExtractId(string url) {
			int i = 0, e = 0;
			while (i < url.Length) {
				if (url[i] >= '1' && url[i] <= '9' && i > 1 && url[i - 1] == '/') {
					e = i;
					i++;
					while (i < url.Length && url[i] >= '0' && url[i] <= '9') {
						i++;
					}
					if (e + 1 < i) {
						return url.Substring(e, i - e);
					}
				}
				i++;
			}
			return null;
		}
		public static string[] GetIds(string value) {
			if (value.Length == 0) {
				return null;
			}
			int i = 0, e = 0, j;
			String t0, t1;
			StringBuilder b = new StringBuilder();
			List<String> r = new List<String>();
			do {
				while (i < value.Length && value[i] <= ' ') {
					i++;
				}
				if (i == value.Length) {
					break;
				} else if (value[i] < '1' || value[i] > '9') {
					j = i;
					i++;
					while (i < value.Length && value[i] > ' ') {
						i++;
					}
					if ((t1 = ExtractId(t0 = value.Substring(j, i - j))) != null) {
						b.Append(t1);
					} else {
						try {
							if ((t1 = GetRedirect(t0)) != null && t1.Length > 0 && t1 != t0 && (t1 = ExtractId(t1)) != null) {
								b.Append(t1);
							}
						} catch (Exception) {
							if (i == value.Length) {
								break;
							}
							i++;
							continue;
						}
					}
					if (i == value.Length) {
						break;
					}
				} else {
					b.Append(value[i]);
					i++;
					while (i < value.Length && value[i] >= '0' && value[i] <= '9') {
						b.Append(value[i]);
						i++;
					}
					if (i == value.Length) {
						break;
					} else if (value[i] > ' ' && (value[i] < '1' || value[i] > '9')) {
						return null;
					}
				}
				if (e == 29) {
					r.Add(b.ToString());
					b.Length = 0;
					e = 0;
				} else {
					b.Append(',');
					e++;
				}
				i++;
			} while (i < value.Length);
			if (b.Length > 0) {
				if (b[b.Length - 1] == ',') {
					b.Length = b.Length - 1;
				}
				r.Add(b.ToString());
			}
			return r.ToArray();
		}
		public string[] GetVideoUrlList(VideoData value) {
			if (Hevc) {
				if (value.PlayAddr265 != null && value.PlayAddr265.UrlList != null && value.PlayAddr265.UrlList.Length > 0) {
					return value.PlayAddr265.UrlList;
				} else if (value.PlayAddrBytevc1 != null && value.PlayAddrBytevc1.UrlList != null && value.PlayAddrBytevc1.UrlList.Length > 0) {
					return value.PlayAddrBytevc1.UrlList;
				}
			}
			if (value.PlayAddr != null && value.PlayAddr.UrlList != null && value.PlayAddr.UrlList.Length > 0) {
				return value.PlayAddr.UrlList;
			}
			return null;
		}
		public void DownloadAweme(object aweme) {
			Aweme a = (Aweme)aweme;
			try {
				bool f = false;
				int r = 0;
				string n, d = a.AddDate && a.Date > 0 ? CreateTimeToFilename(a.Date) : "", t = a.AddDescription && a.Description.Length > 0 ? SanitizeFilename(a.Description) : "";
				byte[] l;
				if (File.Exists(n = (d.Length > 0 ? a.Id + " " + d : a.Id) + (t.Length > 0 ? " " + t + ".mp4" : ".mp4"))) {
					return;
				}
				AwemeWebClient c = new AwemeWebClient();
				do {
					try {
						c.Headers.Set("User-Agent", "okhttp");
						c.DownloadFile(a.Url, n);
						if ((new FileInfo(n)).Length == 0) {
							File.Delete(n);
							throw new Exception("File is empty.");
						}
						break;
					} catch (PathTooLongException) {
						if (t.Length == 0 || f) {
							throw;
						} else if (File.Exists(n = (d.Length > 0 ? a.Id + " " + d : a.Id) + ".mp4")) {
							return;
						}
						f = true;
						r = 0;
					} catch (Exception) {
						if (r == 3) {
							File.Delete(n);
							throw;
						}
						r++;
					}
				} while (true);
				lock (OutLock) {
					Output.AppendText(a.Id);
					Output.AppendText(" is downloaded");
					Output.AppendText(Environment.NewLine);
					if (Logger != null) {
						l = Encoding.UTF8.GetBytes(a.Id + " " + CreateTimeToLog(a.Date) + " " + SanitizeLog(a.Description) + Environment.NewLine);
						Logger.Write(l, 0, l.Length);
					}
				}
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(a.Id);
					Output.AppendText(" failed");
					Output.AppendText(Environment.NewLine);
					Output.AppendText(x.InnerException != null ? x.InnerException.Message : x.Message);
					Output.AppendText(Environment.NewLine);
				}
			} finally {
				lock (SyncLock) {
					Count--;
					Monitor.Pulse(SyncLock);
				}
			}
		}
		public bool RequestAweme(WebClient client, DataContractJsonSerializer serializer, string path, out object result) {
			int r = 0;
			TimeSpan t;
			Stream s;
			Thread.Sleep(1000);
			do {
				lock (StopLock) {
					if (Stop) {
						break;
					}
				}
				t = DateTime.UtcNow.Subtract(new DateTime(1970, 1, 1));
				try {
					client.Headers.Set("User-Agent", "okhttp");
					/*if (Ip.Length > 0) {
						client.Headers.Set("X-Forwarded-For", Ip);
					}*/
					using (s = client.OpenRead("https://api-t2.tiktokv.com" + path
					+ "&region=US"
					//+ "&version_name=10.0.0"
					+ "&ts=" + ((int)t.TotalSeconds).ToString()
					+ "&timezone_name=Etc%2FGMT"
					+ "&device_type=Pixel%20" + DeviceVersion
					+ "&iid=" + IId
					+ "&locale=en"
					+ "&app_type=normal"
					//+ "&build_number=10.0.0"
					+ "&resolution=1080*1920"
					+ "&aid=1180"
					+ "&app_name=musical_ly"
					+ "&_rticket=" + ((long)t.TotalMilliseconds).ToString()
					+ "&device_platform=android"
					+ "&version_code=100000"
					+ "&dpi=441"
					+ "&cpu_support64=false"
					+ "&sys_region=US"
					+ "&timezone_offset=0"
					+ "&device_id=" + DeviceId
					+ "&pass-route=1"
					+ "&device_brand=google"
					+ "&os_version=8.0.0"
					//+ "&ab_version=10.0.0"
					+ "&op_region=US"
					+ "&app_language=en"
					+ "&pass-region=1"
					+ "&language=en"
					+ "&channel=googleplay"
					+ (Hevc ? "&version_name=10.0.0&build_number=10.0.0" : ""))) {
						result = serializer.ReadObject(s);
						return true;
					}
				} catch {
					//Os = GenerateBase62(16);
					//Ip = GeneratePublicIp();
					DeviceId = GenerateDeviceId();
					IId = GenerateDeviceId();
					DeviceVersion = GenerateBase62(8);
					if (r == 32) {//SerializationException or else
						//Ip = "";
						throw;
					}
					r++;
				}
			} while (true);
			result = null;
			return false;
		}
		public void DownloadVideos(object ids) {
			try {
				string d = (string)ids;
				long i = 0;
				int e = 0;
				string[] v, u;
				object r;
				WebClient c = new WebClient();
				DataContractJsonSerializer js = new DataContractJsonSerializer(typeof(AwemesData));
				AwemesData o;
				if ((v = GetIds(d)) == null || v.Length == 0) {
					goto end;
				}
				do {
					if (!RequestAweme(c, js, "/aweme/v1/multi/aweme/detail/?aweme_ids=[" + v[i] + "]", out r)) {//+ "origin_type="//+ "push_params="
						break;
					}
					o = (AwemesData)r;
					if (o != null && o.AwemeDetails != null) {
						for (e = 0; e < o.AwemeDetails.Length; e++) {
							if (o.AwemeDetails[e].Video != null && (u = GetVideoUrlList(o.AwemeDetails[e].Video)) != null) {
								lock (SyncLock) {
									Count++;
								}
								new Thread(new ParameterizedThreadStart(DownloadAweme)).Start(new Aweme {
									Id = o.AwemeDetails[e].AwemeId,
									Date = o.AwemeDetails[e].CreateTime,
									Description = o.AwemeDetails[e].Desc != null ? o.AwemeDetails[e].Desc : "",
									Url = Quality ? GetQualityUri(u[u.Length - 1]) : u[0],
									AddDate = Date,
									AddDescription = Description
								});
								lock (SyncLock) {
									if (Count >= 10) {
										Monitor.Wait(SyncLock);
									}
								}
							}
						}
					}
					i++;
				} while (i < v.Length);
			} catch (Exception x) {
				lock (OutLock) {
					Output.AppendText(x.Message);
					Output.AppendText(Environment.NewLine);
				}
			}
			do {
				lock (SyncLock) {
					if (Count == 0) {
						break;
					}
					Monitor.Wait(SyncLock);
				}
			} while (true);
			end: EnableControls();
		}
		public void AddDateChanged(object sender, EventArgs args) {
			Date = AddDate.Checked;
		}
		public void AddDescriptionChanged(object sender, EventArgs args) {
			Description = AddDescription.Checked;
		}
		public void ImproveQualityChanged(object sender, EventArgs args) {
			Quality = ImproveQuality.Checked;
		}
		public void PreferHevcChanged(object sender, EventArgs args) {
			Hevc = PreferHevc.Checked;
		}
		public void DirLeave(object sender, EventArgs args) {
			try {
				string d0 = GetFullDir(Dir, Root), d1 = GetFullDir(DirInput.Text, Root);
				if (d0 != d1) {
					if (!Directory.Exists(d1)) {
						Directory.CreateDirectory(d1);
					}
					Directory.SetCurrentDirectory(d1);
					Dir = DirInput.Text;
				}
			} catch (Exception x) {
				MessageBox.Show(x.Message);
				DirInput.Text = Dir;
			}
		}
		public void OpenDirClick(object sender, EventArgs args) {
			if (DirOpener.ShowDialog() != DialogResult.OK) {
				return;
			}
			DirInput.Text = DirOpener.SelectedPath;
			DirLeave(DirInput, null);
		}
		public void DownloadClick(object sender, EventArgs args) {
			lock (StopLock) {
				Stop = true;
				if (Input.ReadOnly) {
					return;
				}
			}
			DisableControls();
			if (Log.Checked) {
				try {
					Logger = new FileStream("log.txt", FileMode.Append);
				} catch (Exception x) {
					MessageBox.Show(x.Message);
					EnableControls();
					return;
				}
			}
			if (Input.Text.Length == 0) {
				MessageBox.Show("Input is empty.");
				EnableControls();
				return;
			}
			Output.Text = "";
			new Thread(new ParameterizedThreadStart(DownloadVideos)).Start(Input.Text);
		}
		/*public static bool ValidateRemoteCertificate(object sender, object certificate, object chain, SslPolicyErrors sslPolicyErrors) {
			return true;
		}*/
		[STAThread]
		public static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			ServicePointManager.SecurityProtocol |= SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12 | SecurityProtocolType.Tls13;//.NET 4.8
			//ServicePointManager.ServerCertificateValidationCallback += ValidateRemoteCertificate;//.NET 3.5
			string d = Path.GetDirectoryName(Environment.GetCommandLineArgs()[0]);
			d += d.Length > 0 ? Path.DirectorySeparatorChar + "download" : "download";
			try {
				if (!Directory.Exists(d)) {
					Directory.CreateDirectory(d);
				}
				Directory.SetCurrentDirectory(d);
			} catch {
				MessageBox.Show("Could not initialize download directory.");
				return;
			}
			Application.Run(new MainForm(Path.GetDirectoryName(d), Path.GetFileName(d)));
		}
	}
}