﻿// ==UserScript==
// @name        2chen_mod_log
// @description fun with colours
// @version     2
// @match       https://*.2chen.org/html/mod-log/*
// @match       https://*.2chen.org/html/reports/*
// @match       https://*.2chen.moe/html/mod-log/*
// @match       https://*.2chen.moe/html/reports/*
// @grant       GM.xmlHttpRequest
// ==/UserScript==

(function () {
	'use strict';
	var week = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
	calendar = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
	file_types = ['jpg', 'png', 'gif', 'webm', 'pdf', 'svg', 'mp4', 'mp3', 'ogg', 'zip', '7z', 'tar.gz', 'tar.xz', 'flac', 'no_file', 'txt', 'webp', 'rar', 'cbz', 'cbr', 'mp4 [hevc]'],
	prev_fields,
//	time_relative = false,
	time_relative = true,
	date_now = Date.now(),
	post_links = true,
	post_data = {},
	reports = new Set(),
	reports_page = document.location.href.includes('/reports/'),
	table = document.querySelector('table');
	document.body.style.background = '#222222';
	table.style.background = '#202020';
	table.style.border = '1px solid #cccccc';
	table.style.margin = 'auto';
	Array.from(table.querySelectorAll('tr')).reverse().forEach(function (row) {
		if (row.querySelector('th')) {
			row.style.color = '#cccccc';
			return;
		}
		var [cells, type,by,post,time,data,duration] = (function () {
			var r = [],
			cells = row.querySelectorAll('td');
			r.push(cells);
			cells.forEach(function (cell) {
//				cell.style.border = '1px solid #cccccc';
				r.push(cell.innerText);
			});
			return r;
		})(),
		age = secondsToTime(Math.floor((date_now - (new Date(time))) / 1000)),
		fields = [type,by,time,data,duration].join('|');
		if (prev_fields === fields || (reports_page && reports.has(by))) {
			row.style.color = 'grey';
		} else if (by === 'system' || reports_page) {
			row.style.color = 'white';
		} else if (by === 'admin') {
			row.style.color = 'red';
		} else {
			row.style.color = 'lime';
		}
		prev_fields = fields;
		if (reports_page) reports.add(by); // post id
		cells[3].title = time_relative ? time : age;
		if (time_relative) cells[3].innerText = age;
		if (post_links) {
			var post_cell = cells[reports_page ? 1 : 2],
			next_cell = post_cell.nextSibling,
			post_link = post_cell.querySelector('a[data-id]'),
			post_id = post_link.getAttribute('data-id'),
			post_div = create_div(post_id, {
				background: 'red',
				position: 'absolute',
				display: 'none'
			}),
			post_generated = false;
			post_cell.append(post_div);
			post_link.addEventListener('mouseover', function () {
				var rect = next_cell.getBoundingClientRect();
//				post_div.style.width = rect.width + 'px';
				post_div.style.left = (window.scrollX + rect.left) + 'px';
				post_div.style.top = (window.scrollY + rect.top) + 'px';
				post_div.style.display = 'block';
				if (post_generated) return;
				load_post(post_id, function (data) {
					generate_post(post_div, JSON.parse(data));
					post_generated = true;
				});
			});
			post_link.addEventListener('mouseout', function () {
				post_div.style.display = 'none';
			});
		}
	});
	function generate_post(link, data) {
		var time = new Date(data.time*1000),
		age = secondsToTime(Math.floor((date_now - time) / 1000)),
		post = create_el('article', {
			display: 'flex',
			flexDirection: 'column',
			alignItems: 'flex-start',
			background: '#2f343e',
			color: '#c5c8c6'
		}),
		post_header = create_el('header', {
			display: 'block',
			padding: '10px'
		}),
		post_container = create_el('div');
		post_header.append(create_div('No.' + data.id, {color: '#81a2be'}));
		post_header.append(create_div([readableTime(time),age].join(' \u2014 ')));
		if (data.image !== null) {
			var image = create_el('img'),
			image_container = create_el('figure', {
				float: 'left',
				margin: '5px 10px 0 0',
				padding: '10px'
			});
			image.src = '/assets/images/thumb/' + data.image.sha1 + '.jpg';
			image_container.append(image);
			post_container.append(image_container);
			post_header.append(create_div(data.image.name + '.' + file_type(data.image.file_type), {color: '#81a2be'}));
			['artist', 'title'].filter(p => data.image[p] !== '').forEach(p => post_header.append(create_div(p + ': ' + data.image[p])));
		}
		if (data.moderation !== null) {
			post_header.append(create_el('br'));
			data.moderation.forEach(function (action) {
				post_header.append(create_div(action_text(action), {
					color: '#f00000',
					fontWeight: 'bold'
				}));
			});
		}
		post_container.append(create_blockquote(data.body, {minWidth: '25em'}));
		post.append(post_header);
		post.append(post_container);
		link.style.background = '';
		link.innerHTML = post.outerHTML;
	}
	function load_post(post_id, callback) {
		if (post_data.hasOwnProperty(post_id)) {
			callback(post_data[post_id]);
		} else {
			GM.xmlHttpRequest({
				method: 'GET',
				url: 'https://2chen.moe/json/post/' + post_id,
				onload: function (response) {
					if (response.status === 200) {
						callback(post_data[post_id] = response.responseText);
					} else {
						console.error(response);
					}
				}
			});
		}
	}
	function create_el(el_tag, styles) {
		var el = document.createElement(el_tag);
		if (styles) {
			for (var [property, value] of Object.entries(styles)) {
				el.style[property] = value;
			}
		}
		return el;
	}
	function create_div(v, styles) {
		var el = create_el('div', styles);
		el.append(document.createTextNode(v));
		return el;
	}
	function create_blockquote(v, styles) {
		var el = create_el('blockquote', styles);
		el.innerText = v;
		return el;
	}
	function file_type(v) {
		return file_types.hasOwnProperty(v) ? file_types[v] : '?';
	}
	function action_text(action) {
		switch (action.type) {
//			https://2chen.moe/tv/5#p834
			case 0: return 'BANNED BY "' + action.by + '" FOR ' + secondsToTime(action.length, false) + ' FOR "' + action.data + '"';
//			https://2chen.moe/tv/4025#p5618
			case 1: return 'UNBANNED BY "' + action.by + '"';
//			https://2chen.moe/tv/5#p1889
			case 2: return 'DELETED BY "' + action.by + '"';
//			https://2chen.moe/tv/5#p1889
			case 3: return 'IMAGE DELETED BY "' + action.by + '"';
//			https://2chen.moe/tv/2024#p3090
			case 4: return 'IMAGE SPOILERED BY "' + action.by + '"';
//			https://2chen.moe/tv/1#p1
			case 5: return 'THREAD LOCKED BY "' + action.by + '"';
//			??
//			case 6: return '?? BY "' + action.by + '"';
//			https://2chen.moe/tv/5#p1847
			case 7: return 'POSTS OF THE SAME IP WERE VIEWED BY "' + action.by + '"';
//			https://2chen.moe/tv/5014#p5014
			case 8: return 'POST PURGED BY "' + action.by + '" FOR "' + action.data + '"';
//			https://2chen.moe/tv/4025#p4679
			case 9: return 'SHADOW BINNED BY "' + action.by + '" FOR ' + secondsToTime(action.length, false) + ' FOR "' + action.data + '"';
//			https://2chen.moe/tv/1375636#p1898706
			case 10: return 'POSTING BANNED BY "' + action.by + '" FOR ' + secondsToTime(action.length, false) + ' FOR "' + action.data + '"';
			default: return JSON.stringify(action);
		}
	}
	function pad(n) {
		return (n < 10 ? '0' : '') + n;
	}
	function readableTime(d) {
		return `${pad(d.getDate())} ${calendar[d.getMonth()]} `
		+ `${d.getFullYear()} (${week[d.getDay()]}) `
		+ `${pad(d.getHours())}:${pad(d.getMinutes())}:${pad(d.getSeconds())}`;
	}
	function secondsToTime(s, relative=true) {
		if (s === 0) {
			return 'just now';
		}
		const units = ['second', 'minute', 'hour', 'day', 'month', 'year'];
		const divide = [60, 60, 24, 30, 12];
		let interval = [1];
		for (let i = 0; i < divide.length; i++) {
			interval.push(interval[i] * divide[i]);
		}
		interval[5] = interval[3] * 365; // year = day * 365
		let time = s;
		let format = [];
		for (let i = interval.length; i >= 0; i--) {
			if (time >= interval[i]) {
				let unit = Math.floor(time / interval[i]);
				format.push(unit + ' ' + units[i] + (unit === 1 ? '' : 's'));
				time -= unit * interval[i];
			}
		}
		format.splice(2);
		return format.join(', ') + (relative ? ' ago' : '');
	}
})();